@@ -1,17 +1,13 @@
-/** @file	eggtrayicon.c
- *  @brief	EggTrayIcon low level hooks.
- *  @author	Anders Carlsson <andersca@gnu.org>
- *  @date	2002
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
+ * Copyright (C) 2002 Anders Carlsson <andersca@gnu.org>
  *
- *  @todo	Don't use this file. Use the sexy new stuff in CVS gtk.
- */
-/*
  * Licensed under the GNU General Public License Version 2
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,20 +16,22 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
-
 #include <config.h>
 #include <string.h>
 #include <libintl.h>
 
 #include "eggtrayicon.h"
 
+#include <gdkconfig.h>
+#if defined (GDK_WINDOWING_X11)
 #include <gdk/gdkx.h>
 #include <X11/Xatom.h>
+#elif defined (GDK_WINDOWING_WIN32)
+#include <gdk/gdkwin32.h>
+#endif
 
 #ifndef EGG_COMPILATION
 #ifndef _
@@ -45,9 +43,9 @@
 #define N_(x) x
 #endif
 
-#define SYSTEM_TRAY_REQUEST_DOCK		0
-#define SYSTEM_TRAY_BEGIN_MESSAGE	 1
-#define SYSTEM_TRAY_CANCEL_MESSAGE	2
+#define SYSTEM_TRAY_REQUEST_DOCK    0
+#define SYSTEM_TRAY_BEGIN_MESSAGE   1
+#define SYSTEM_TRAY_CANCEL_MESSAGE  2
 
 #define SYSTEM_TRAY_ORIENTATION_HORZ 0
 #define SYSTEM_TRAY_ORIENTATION_VERT 1
@@ -59,18 +57,22 @@ enum {
 
 static GtkPlugClass *parent_class = NULL;
 
-static void egg_tray_icon_init (EggTrayIcon * icon);
-static void egg_tray_icon_class_init (EggTrayIconClass * klass);
+static void egg_tray_icon_init (EggTrayIcon *icon);
+static void egg_tray_icon_class_init (EggTrayIconClass *klass);
 
-static void egg_tray_icon_get_property (GObject * object,
-					guint prop_id,
-					GValue * value,
-					GParamSpec * pspec);
+static void egg_tray_icon_get_property (GObject    *object,
+					guint       prop_id,
+					GValue     *value,
+					GParamSpec *pspec);
 
-static void egg_tray_icon_realize (GtkWidget * widget);
-static void egg_tray_icon_unrealize (GtkWidget * widget);
+static void egg_tray_icon_realize   (GtkWidget *widget);
+static void egg_tray_icon_unrealize (GtkWidget *widget);
 
-static void egg_tray_icon_update_manager_window (EggTrayIcon * icon);
+#ifdef GDK_WINDOWING_X11
+static void egg_tray_icon_update_manager_window (EggTrayIcon *icon,
+							gboolean dock_if_realized);
+static void egg_tray_icon_manager_window_destroyed (EggTrayIcon *icon);
+#endif
 
 GType
 egg_tray_icon_get_type (void)
@@ -89,30 +91,26 @@ egg_tray_icon_get_type (void)
 			0,	/* n_preallocs */
 			(GInstanceInitFunc) egg_tray_icon_init
 		};
-
-		our_type =
-		    g_type_register_static (GTK_TYPE_PLUG, "EggTrayIcon",
-					    &our_info, 0);
+		our_type = g_type_register_static (GTK_TYPE_PLUG, "EggTrayIcon", &our_info, 0);
 	}
 
 	return our_type;
 }
 
 static void
-egg_tray_icon_init (EggTrayIcon * icon)
+egg_tray_icon_init (EggTrayIcon *icon)
 {
 	icon->stamp = 1;
 	icon->orientation = GTK_ORIENTATION_HORIZONTAL;
 
-	gtk_widget_add_events (GTK_WIDGET (icon),
-			       GDK_PROPERTY_CHANGE_MASK);
+	gtk_widget_add_events (GTK_WIDGET (icon), GDK_PROPERTY_CHANGE_MASK);
 }
 
 static void
-egg_tray_icon_class_init (EggTrayIconClass * klass)
+egg_tray_icon_class_init (EggTrayIconClass *klass)
 {
-	GObjectClass *gobject_class = (GObjectClass *) klass;
-	GtkWidgetClass *widget_class = (GtkWidgetClass *) klass;
+	GObjectClass *gobject_class = (GObjectClass *)klass;
+	GtkWidgetClass *widget_class = (GtkWidgetClass *)klass;
 
 	parent_class = g_type_class_peek_parent (klass);
 
@@ -124,32 +122,42 @@ egg_tray_icon_class_init (EggTrayIconClass * klass)
 	g_object_class_install_property (gobject_class,
 					 PROP_ORIENTATION,
 					 g_param_spec_enum ("orientation",
-							    _("Orientation"),
-							    _("The orientation of the tray."),
-							    GTK_TYPE_ORIENTATION,
-							    GTK_ORIENTATION_HORIZONTAL,
-							    G_PARAM_READABLE));
+					 _("Orientation"),
+					 _("The orientation of the tray."),
+					 GTK_TYPE_ORIENTATION,
+					 GTK_ORIENTATION_HORIZONTAL,
+					 G_PARAM_READABLE));
+
+#if defined (GDK_WINDOWING_X11)
+	/* Nothing */
+#elif defined (GDK_WINDOWING_WIN32)
+	g_warning ("Port eggtrayicon to Win32");
+#else
+	g_warning ("Port eggtrayicon to this GTK+ backend");
+#endif
 }
 
 static void
-egg_tray_icon_get_property (GObject * object,
+egg_tray_icon_get_property (GObject *object,
 			    guint prop_id,
-			    GValue * value, GParamSpec * pspec)
+			    GValue *value,
+			    GParamSpec *pspec)
 {
 	EggTrayIcon *icon = EGG_TRAY_ICON (object);
 
 	switch (prop_id) {
-	case PROP_ORIENTATION:
-		g_value_set_enum (value, icon->orientation);
-		break;
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
+		case PROP_ORIENTATION:
+			g_value_set_enum (value, icon->orientation);
+			break;
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+			break;
 	}
 }
 
+#ifdef GDK_WINDOWING_X11
 static void
-egg_tray_icon_get_orientation_property (EggTrayIcon * icon)
+egg_tray_icon_get_orientation_property (EggTrayIcon *icon)
 {
 	Display *xdisplay;
 	Atom type;
@@ -157,17 +165,14 @@ egg_tray_icon_get_orientation_property (EggTrayIcon * icon)
 	union {
 		gulong *prop;
 		guchar *prop_ch;
-	} prop = {
-	NULL};
+	} prop = { NULL };
 	gulong nitems;
 	gulong bytes_after;
 	int error, result;
 
 	g_assert (icon->manager_window != None);
 
-	xdisplay =
-	    GDK_DISPLAY_XDISPLAY (gtk_widget_get_display
-				  (GTK_WIDGET (icon)));
+	xdisplay = GDK_DISPLAY_XDISPLAY (gtk_widget_get_display (GTK_WIDGET (icon)));
 
 	gdk_error_trap_push ();
 	type = None;
@@ -178,6 +183,7 @@ egg_tray_icon_get_orientation_property (EggTrayIcon * icon)
 				     XA_CARDINAL,
 				     &type, &format, &nitems,
 				     &bytes_after, &(prop.prop_ch));
+
 	error = gdk_error_trap_pop ();
 
 	if (error || result != Success)
@@ -186,14 +192,12 @@ egg_tray_icon_get_orientation_property (EggTrayIcon * icon)
 	if (type == XA_CARDINAL) {
 		GtkOrientation orientation;
 
-		orientation =
-		    (prop.prop[0] ==
-		     SYSTEM_TRAY_ORIENTATION_HORZ) ?
-		    GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL;
+		orientation = (prop.prop [0] == SYSTEM_TRAY_ORIENTATION_HORZ) ?
+			       GTK_ORIENTATION_HORIZONTAL :
+			       GTK_ORIENTATION_VERTICAL;
 
 		if (icon->orientation != orientation) {
 			icon->orientation = orientation;
-
 			g_object_notify (G_OBJECT (icon), "orientation");
 		}
 	}
@@ -203,60 +207,58 @@ egg_tray_icon_get_orientation_property (EggTrayIcon * icon)
 }
 
 static GdkFilterReturn
-egg_tray_icon_manager_filter (GdkXEvent * xevent, GdkEvent * event,
-			      gpointer user_data)
+egg_tray_icon_manager_filter (GdkXEvent *xevent, GdkEvent *event, gpointer user_data)
 {
 	EggTrayIcon *icon = user_data;
-	XEvent *xev = (XEvent *) xevent;
+	XEvent *xev = (XEvent *)xevent;
 
 	if (xev->xany.type == ClientMessage &&
 	    xev->xclient.message_type == icon->manager_atom &&
 	    xev->xclient.data.l[1] == icon->selection_atom) {
-		egg_tray_icon_update_manager_window (icon);
+		egg_tray_icon_update_manager_window (icon, TRUE);
 	} else if (xev->xany.window == icon->manager_window) {
 		if (xev->xany.type == PropertyNotify &&
 		    xev->xproperty.atom == icon->orientation_atom) {
 			egg_tray_icon_get_orientation_property (icon);
 		}
 		if (xev->xany.type == DestroyNotify) {
-			egg_tray_icon_update_manager_window (icon);
+			egg_tray_icon_manager_window_destroyed (icon);
 		}
 	}
 
 	return GDK_FILTER_CONTINUE;
 }
+#endif
 
 static void
-egg_tray_icon_unrealize (GtkWidget * widget)
+egg_tray_icon_unrealize (GtkWidget *widget)
 {
+#ifdef GDK_WINDOWING_X11
 	EggTrayIcon *icon = EGG_TRAY_ICON (widget);
 	GdkWindow *root_window;
 
 	if (icon->manager_window != None) {
 		GdkWindow *gdkwin;
 
-		gdkwin =
-		    gdk_window_lookup_for_display (gtk_widget_get_display
-						   (widget),
-						   icon->manager_window);
+		gdkwin = gdk_window_lookup_for_display (gtk_widget_get_display (widget),
+						icon->manager_window);
 
-		gdk_window_remove_filter (gdkwin,
-					  egg_tray_icon_manager_filter,
-					  icon);
+		gdk_window_remove_filter (gdkwin, egg_tray_icon_manager_filter, icon);
 	}
 
-	root_window =
-	    gdk_screen_get_root_window (gtk_widget_get_screen (widget));
+	root_window = gdk_screen_get_root_window (gtk_widget_get_screen (widget));
 
-	gdk_window_remove_filter (root_window,
-				  egg_tray_icon_manager_filter, icon);
+	gdk_window_remove_filter (root_window, egg_tray_icon_manager_filter, icon);
 
 	if (GTK_WIDGET_CLASS (parent_class)->unrealize)
 		(*GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
+#endif
 }
 
+#ifdef GDK_WINDOWING_X11
+
 static void
-egg_tray_icon_send_manager_message (EggTrayIcon * icon,
+egg_tray_icon_send_manager_message (EggTrayIcon *icon,
 				    long message,
 				    Window window,
 				    long data1, long data2, long data3)
@@ -274,50 +276,34 @@ egg_tray_icon_send_manager_message (EggTrayIcon * icon,
 	ev.data.l[3] = data2;
 	ev.data.l[4] = data3;
 
-	display =
-	    GDK_DISPLAY_XDISPLAY (gtk_widget_get_display
-				  (GTK_WIDGET (icon)));
+	display = GDK_DISPLAY_XDISPLAY (gtk_widget_get_display (GTK_WIDGET (icon)));
 
 	gdk_error_trap_push ();
 	XSendEvent (display,
-		    icon->manager_window, False, NoEventMask,
-		    (XEvent *) & ev);
+		    icon->manager_window, False, NoEventMask, (XEvent *)&ev);
 	XSync (display, False);
 	gdk_error_trap_pop ();
 }
 
 static void
-egg_tray_icon_send_dock_request (EggTrayIcon * icon)
+egg_tray_icon_send_dock_request (EggTrayIcon *icon)
 {
 	egg_tray_icon_send_manager_message (icon,
 					    SYSTEM_TRAY_REQUEST_DOCK,
 					    icon->manager_window,
-					    gtk_plug_get_id (GTK_PLUG
-							     (icon)), 0,
-					    0);
+					    gtk_plug_get_id (GTK_PLUG (icon)),
+					    0, 0);
 }
 
 static void
-egg_tray_icon_update_manager_window (EggTrayIcon * icon)
+egg_tray_icon_update_manager_window (EggTrayIcon *icon, gboolean dock_if_realized)
 {
 	Display *xdisplay;
 
-	xdisplay =
-	    GDK_DISPLAY_XDISPLAY (gtk_widget_get_display
-				  (GTK_WIDGET (icon)));
+	if (icon->manager_window != None)
+		return;
 
-	if (icon->manager_window != None) {
-		GdkWindow *gdkwin;
-
-		gdkwin =
-		    gdk_window_lookup_for_display (gtk_widget_get_display
-						   (GTK_WIDGET (icon)),
-						   icon->manager_window);
-
-		gdk_window_remove_filter (gdkwin,
-					  egg_tray_icon_manager_filter,
-					  icon);
-	}
+	xdisplay = GDK_DISPLAY_XDISPLAY (gtk_widget_get_display (GTK_WIDGET (icon)));
 
 	XGrabServer (xdisplay);
 
@@ -327,7 +313,7 @@ egg_tray_icon_update_manager_window (EggTrayIcon * icon)
 	if (icon->manager_window != None)
 		XSelectInput (xdisplay,
 			      icon->manager_window,
-			      StructureNotifyMask | PropertyChangeMask);
+			      StructureNotifyMask|PropertyChangeMask);
 
 	XUngrabServer (xdisplay);
 	XFlush (xdisplay);
@@ -335,24 +321,41 @@ egg_tray_icon_update_manager_window (EggTrayIcon * icon)
 	if (icon->manager_window != None) {
 		GdkWindow *gdkwin;
 
-		gdkwin =
-		    gdk_window_lookup_for_display (gtk_widget_get_display
-						   (GTK_WIDGET (icon)),
-						   icon->manager_window);
+		gdkwin = gdk_window_lookup_for_display (gtk_widget_get_display (GTK_WIDGET (icon)),
+						        icon->manager_window);
 
-		gdk_window_add_filter (gdkwin,
-				       egg_tray_icon_manager_filter, icon);
+		gdk_window_add_filter (gdkwin, egg_tray_icon_manager_filter, icon);
 
-		/* Send a request that we'd like to dock */
-		egg_tray_icon_send_dock_request (icon);
+		if (dock_if_realized && GTK_WIDGET_REALIZED (icon))
+			egg_tray_icon_send_dock_request (icon);
 
 		egg_tray_icon_get_orientation_property (icon);
 	}
 }
 
 static void
-egg_tray_icon_realize (GtkWidget * widget)
+egg_tray_icon_manager_window_destroyed (EggTrayIcon *icon)
 {
+	GdkWindow *gdkwin;
+
+	g_return_if_fail (icon->manager_window != None);
+
+	gdkwin = gdk_window_lookup_for_display (gtk_widget_get_display (GTK_WIDGET (icon)),
+						icon->manager_window);
+
+	gdk_window_remove_filter (gdkwin, egg_tray_icon_manager_filter, icon);
+
+	icon->manager_window = None;
+
+	egg_tray_icon_update_manager_window (icon, TRUE);
+}
+
+#endif
+
+static void
+egg_tray_icon_realize (GtkWidget *widget)
+{
+#ifdef GDK_WINDOWING_X11
 	EggTrayIcon *icon = EGG_TRAY_ICON (widget);
 	GdkScreen *screen;
 	GdkDisplay *display;
@@ -384,68 +387,53 @@ egg_tray_icon_realize (GtkWidget * widget)
 					      "_NET_SYSTEM_TRAY_ORIENTATION",
 					      False);
 
-	egg_tray_icon_update_manager_window (icon);
+	egg_tray_icon_update_manager_window (icon, FALSE);
+	egg_tray_icon_send_dock_request (icon);
 
 	root_window = gdk_screen_get_root_window (screen);
 
 	/* Add a root window filter so that we get changes on MANAGER */
-	gdk_window_add_filter (root_window,
-			       egg_tray_icon_manager_filter, icon);
+	gdk_window_add_filter (root_window, egg_tray_icon_manager_filter, icon);
+#endif
 }
 
 EggTrayIcon *
-egg_tray_icon_new_for_xscreen (Screen * xscreen, const char *name)
-{
-	GdkDisplay *display;
-	GdkScreen *screen;
-
-	display = gdk_x11_lookup_xdisplay (DisplayOfScreen (xscreen));
-	screen =
-	    gdk_display_get_screen (display,
-				    XScreenNumberOfScreen (xscreen));
-
-	return egg_tray_icon_new_for_screen (screen, name);
-}
-
-EggTrayIcon *
-egg_tray_icon_new_for_screen (GdkScreen * screen, const char *name)
+egg_tray_icon_new_for_screen (GdkScreen *screen, const char *name)
 {
 	g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
 
-	return g_object_new (EGG_TYPE_TRAY_ICON, "screen", screen, "title",
-			     name, NULL);
+	return g_object_new (EGG_TYPE_TRAY_ICON, "screen", screen, "title", name, NULL);
 }
 
-EggTrayIcon *
-egg_tray_icon_new (const gchar * name)
+EggTrayIcon*
+egg_tray_icon_new (const gchar *name)
 {
 	return g_object_new (EGG_TYPE_TRAY_ICON, "title", name, NULL);
 }
 
 guint
-egg_tray_icon_send_message (EggTrayIcon * icon,
-			    gint timeout, const gchar * message, gint len)
+egg_tray_icon_send_message (EggTrayIcon *icon, gint timeout, const gchar *message, gint len)
 {
 	guint stamp;
 
 	g_return_val_if_fail (EGG_IS_TRAY_ICON (icon), 0);
 	g_return_val_if_fail (timeout >= 0, 0);
 	g_return_val_if_fail (message != NULL, 0);
-
+				
+#ifdef GDK_WINDOWING_X11
 	if (icon->manager_window == None)
 		return 0;
+#endif
 
 	if (len < 0)
 		len = strlen (message);
 
 	stamp = icon->stamp++;
 
+#ifdef GDK_WINDOWING_X11
 	/* Get ready to send the message */
-	egg_tray_icon_send_manager_message (icon,
-					    SYSTEM_TRAY_BEGIN_MESSAGE,
-					    (Window)
-					    gtk_plug_get_id (GTK_PLUG
-							     (icon)),
+	egg_tray_icon_send_manager_message (icon, SYSTEM_TRAY_BEGIN_MESSAGE,
+					    (Window)gtk_plug_get_id (GTK_PLUG (icon)),
 					    timeout, len, stamp);
 
 	/* Now to send the actual message */
@@ -454,12 +442,10 @@ egg_tray_icon_send_message (EggTrayIcon * icon,
 		XClientMessageEvent ev;
 		Display *xdisplay;
 
-		xdisplay =
-		    GDK_DISPLAY_XDISPLAY (gtk_widget_get_display
-					  (GTK_WIDGET (icon)));
+		xdisplay = GDK_DISPLAY_XDISPLAY (gtk_widget_get_display (GTK_WIDGET (icon)));
 
 		ev.type = ClientMessage;
-		ev.window = (Window) gtk_plug_get_id (GTK_PLUG (icon));
+		ev.window = (Window)gtk_plug_get_id (GTK_PLUG (icon));
 		ev.format = 8;
 		ev.message_type = XInternAtom (xdisplay,
 					       "_NET_SYSTEM_TRAY_MESSAGE_DATA",
@@ -474,36 +460,31 @@ egg_tray_icon_send_message (EggTrayIcon * icon,
 		}
 
 		XSendEvent (xdisplay,
-			    icon->manager_window, False,
-			    StructureNotifyMask, (XEvent *) & ev);
+		icon->manager_window, False, StructureNotifyMask, (XEvent *)&ev);
 		XSync (xdisplay, False);
 	}
 	gdk_error_trap_pop ();
+#endif
 
 	return stamp;
 }
 
 void
-egg_tray_icon_cancel_message (EggTrayIcon * icon, guint id)
+egg_tray_icon_cancel_message (EggTrayIcon *icon, guint id)
 {
 	g_return_if_fail (EGG_IS_TRAY_ICON (icon));
 	g_return_if_fail (id > 0);
-
-	egg_tray_icon_send_manager_message (icon,
-					    SYSTEM_TRAY_CANCEL_MESSAGE,
-					    (Window)
-					    gtk_plug_get_id (GTK_PLUG
-							     (icon)), id,
-					    0, 0);
+#ifdef GDK_WINDOWING_X11
+	egg_tray_icon_send_manager_message (icon, SYSTEM_TRAY_CANCEL_MESSAGE,
+							(Window)gtk_plug_get_id (GTK_PLUG (icon)),
+							id, 0, 0);
+#endif
 }
 
 GtkOrientation
-egg_tray_icon_get_orientation (EggTrayIcon * icon)
+egg_tray_icon_get_orientation (EggTrayIcon *icon)
 {
-	g_return_val_if_fail (EGG_IS_TRAY_ICON (icon),
-			      GTK_ORIENTATION_HORIZONTAL);
+	g_return_val_if_fail (EGG_IS_TRAY_ICON (icon), GTK_ORIENTATION_HORIZONTAL);
 
 	return icon->orientation;
 }
-
-#endif /* DOXYGEN_SHOULD_SKIP_THIS */
