@@ -1,20 +1,13 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- *  @file	gpm-common.c
- *  @brief	Common functions shared between modules
- *  @author	Richard Hughes <richard@hughsie.com>
- *  @date	2005-10-02
- *
- * This module contains functions that are shared between g-p-m and
- * g-p-m so that as much code can be re-used as possible.
- * There's a bit of everything in this file...
+ * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,8 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
@@ -36,19 +28,23 @@
 
 /** Returns the time string, e.g. "2 hours 3 minutes"
  *
- *  @param  minutes		Minutes to convert to string
+ *  @param  time		Time to convert to string (in seconds)
  *  @return			The timestring
  *
- *  @note	minutes == 0 is returned as "Unknown"
+ *  @note	time == 0 is returned as "Unknown"
  */
 char *
-get_timestring_from_minutes (int minutes)
+gpm_get_timestring (int time)
 {
 	char* timestring = NULL;
-	gint   hours;
+	gint  hours;
+	gint  minutes;
+
+	/* Add 0.5 to do rounding */
+	minutes = (int) ( ( time / 60.0 ) + 0.5 );
 
 	if (minutes == 0) {
-		timestring = g_strdup_printf (_("Unknown"));
+		timestring = g_strdup_printf (_("Unknown time"));
 		return timestring;
 	}
 
@@ -62,7 +58,7 @@ get_timestring_from_minutes (int minutes)
 	hours = minutes / 60;
 	minutes = minutes % 60;
 
-	if (minutes == 0) 
+	if (minutes == 0)
 		timestring = g_strdup_printf (ngettext (
 				"%i hour",
 				"%i hours",
