@@ -59,8 +59,8 @@
 #include "gpm-manager.h"
 
 static void     gpm_manager_class_init (GpmManagerClass *klass);
-static void     gpm_manager_init       (GpmManager      *manager);
-static void     gpm_manager_finalize   (GObject         *object);
+static void     gpm_manager_init	(GpmManager      *manager);
+static void     gpm_manager_finalize   (GObject	  *object);
 
 #define GPM_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_MANAGER, GpmManagerPrivate))
 
@@ -83,8 +83,7 @@ typedef enum {
 #define GPM_BUTTON_BRIGHT_UP_DEP	"brightness_up"		/* Remove when we depend on HAL 0.5.8 */
 #define GPM_BUTTON_BRIGHT_DOWN_DEP	"brightness_down"	/* as these are the old names */
 #define GPM_BUTTON_LOCK			"lock"
-/* Using www until we get a better one defined for us by the kernel */
-#define GPM_BUTTON_BATTERY		"www"
+#define GPM_BUTTON_BATTERY		"battery"
 
 #define GPM_NOTIFY_TIMEOUT_LONG		20	/* seconds */
 #define GPM_NOTIFY_TIMEOUT_SHORT	5	/* seconds */
@@ -110,9 +109,12 @@ struct GpmManagerPrivate
 
 	gboolean	 use_time_to_notify;
 	gboolean	 lid_is_closed;
+	gboolean	 done_notify_fully_charged;
+
+	time_t           last_resume_event;
+	int		 suppress_policy_timeout;
 
 	const char	*reason;
-
 };
 
 enum {
@@ -129,11 +131,12 @@ static GObjectClass *parent_class = NULL;
 static guint	     signals [LAST_SIGNAL] = { 0, };
 
 static GConfEnumStringPair icon_policy_enum_map [] = {
-       { GPM_ICON_POLICY_ALWAYS,       "always"   },
-       { GPM_ICON_POLICY_CHARGE,       "charge"   },
-       { GPM_ICON_POLICY_CRITICAL,     "critical" },
-       { GPM_ICON_POLICY_NEVER,        "never"    },
-       { 0, NULL }
+	{ GPM_ICON_POLICY_ALWAYS,	"always"   },
+	{ GPM_ICON_POLICY_PRESENT,	"present"  },
+	{ GPM_ICON_POLICY_CHARGE,	"charge"   },
+	{ GPM_ICON_POLICY_CRITICAL,     "critical" },
+	{ GPM_ICON_POLICY_NEVER,	"never"    },
+	{ 0, NULL }
 };
 
 G_DEFINE_TYPE (GpmManager, gpm_manager, G_TYPE_OBJECT)
@@ -170,6 +173,21 @@ gpm_manager_error_quark (void)
 	return quark;
 }
 
+/* Returns if an action is valid, i.e. if the difference in time between this
+   request for an action, and the last action completing is larger than the
+   timeout set in gconf. This should fix lots of ACPI bugs we are having. */
+static gboolean
+gpm_manager_is_policy_timout_valid (GpmManager *manager,
+				    const char *action)
+{
+	if ((time (NULL) - manager->priv->last_resume_event) <=
+	    manager->priv->suppress_policy_timeout) {
+		gpm_debug ("Skipping suppressed %s", action);
+		return FALSE;
+	}
+	return TRUE;
+}
+
 gboolean
 gpm_manager_can_suspend (GpmManager *manager,
 			 gboolean   *can,
@@ -302,7 +320,7 @@ get_stock_id_helper (GpmPowerBatteryStatus *device_status, const char *prefix)
 /* must free retval */
 static char *
 get_stock_id (GpmManager *manager,
-	      int         icon_policy)
+	      int	  icon_policy)
 {
 	GpmPowerBatteryStatus status_primary;
 	GpmPowerBatteryStatus status_ups;
@@ -310,7 +328,6 @@ get_stock_id (GpmManager *manager,
 	GpmPowerBatteryStatus status_keyboard;
 	gboolean on_ac;
 	gboolean present;
-	gboolean use_fallback;
 
 	gpm_debug ("Getting stock icon for tray");
 
@@ -391,19 +408,15 @@ get_stock_id (GpmManager *manager,
 		return get_stock_id_helper (&status_ups, ICON_PREFIX_UPS);
 	}
 
-	/* we have a falback here, so that desktops by default do not have
-	   the icon, but it can be "forced" */
-	use_fallback = gconf_client_get_bool (manager->priv->gconf_client,
-					      GPM_PREF_USE_ICON_DESKTOPS,
-					      NULL);
-	if (use_fallback) {
-		/* we fallback to the ac_adapter icon */
-		gpm_debug ("Using fallback");
-		return g_strdup_printf (GPM_STOCK_AC_ADAPTER);
+	/* Check if we should just fallback to the ac icon */
+	if (icon_policy == GPM_ICON_POLICY_PRESENT) {
+		gpm_debug ("no devices present, so no icon will be displayed.");
+		return NULL;
 	}
-	/* no icon! */
-	gpm_debug ("No devices present, and no fallback, so no icon displayed.");
-	return NULL;
+
+	/* we fallback to the ac_adapter icon */
+	gpm_debug ("Using fallback");
+	return g_strdup_printf (GPM_STOCK_AC_ADAPTER);
 }
 
 static void
@@ -576,7 +589,7 @@ manager_do_we_screensave (GpmManager *manager,
 	} else {
 		do_lock = gconf_client_get_bool (manager->priv->gconf_client,
 						 policy, NULL);
-		gpm_debug ("Using constom locking settings (%i)", do_lock);
+		gpm_debug ("Using custom locking settings (%i)", do_lock);
 	}
 	return do_lock;
 }
@@ -591,7 +604,8 @@ gpm_manager_blank_screen (GpmManager *manager,
 	do_lock = manager_do_we_screensave (manager,
 					    GPM_PREF_LOCK_ON_BLANK_SCREEN);
 	if (do_lock) {
-		gpm_screensaver_lock ();
+		if (!gpm_screensaver_lock ())
+			gpm_debug ("Could not lock screen via gnome-screensaver");
 	}
 	/* We give the options to enable DPMS because some laptops do
 	 * not turn off the LCD backlight when the lid is closed. 
@@ -644,6 +658,10 @@ manager_policy_do (GpmManager *manager,
 		return;
 	}
 
+	if (! gpm_manager_is_policy_timout_valid (manager, "policy event")) {
+		return;
+	}
+
 	if (strcmp (action, ACTION_NOTHING) == 0) {
 
 		gpm_debug ("*ACTION* Doing nothing");
@@ -685,8 +703,8 @@ manager_policy_do (GpmManager *manager,
 
 gboolean
 gpm_manager_get_on_ac (GpmManager *manager,
-		       gboolean	  *on_ac,
-		       GError    **error)
+			gboolean	  *on_ac,
+			GError    **error)
 {
 	g_return_val_if_fail (GPM_IS_MANAGER (manager), FALSE);
 
@@ -784,7 +802,7 @@ gpm_manager_shutdown (GpmManager *manager,
 
 static gboolean
 gpm_manager_hibernate (GpmManager *manager,
-		       GError    **error)
+			GError    **error)
 {
 	gboolean allowed;
 	gboolean ret;
@@ -836,6 +854,11 @@ gpm_manager_hibernate (GpmManager *manager,
 	}
 	gpm_networkmanager_wake ();
 
+	sync_dpms_policy (manager);
+
+	/* save the time that we resumed */
+	manager->priv->last_resume_event = time (NULL);
+
 	return ret;
 }
 
@@ -893,6 +916,11 @@ gpm_manager_suspend (GpmManager *manager,
 	}
 	gpm_networkmanager_wake ();
 
+	sync_dpms_policy (manager);
+
+	/* save the time that we resumed */
+	manager->priv->last_resume_event = time (NULL);
+
 	return ret;
 }
 
@@ -954,7 +982,7 @@ idle_changed_cb (GpmIdle    *idle,
 
 		/* Should we dim the screen? */
 		do_laptop_dim = gconf_client_get_bool (manager->priv->gconf_client,
-						       GPM_PREF_IDLE_DIM_SCREEN, NULL);
+							GPM_PREF_IDLE_DIM_SCREEN, NULL);
 		if (do_laptop_dim) {
 			/* resume to the previous brightness */
 			gpm_brightness_level_resume (manager->priv->brightness);
@@ -977,7 +1005,7 @@ idle_changed_cb (GpmIdle    *idle,
 
 		/* Should we resume the screen? */
 		do_laptop_dim = gconf_client_get_bool (manager->priv->gconf_client,
-						       GPM_PREF_IDLE_DIM_SCREEN, NULL);
+							GPM_PREF_IDLE_DIM_SCREEN, NULL);
 		if (do_laptop_dim) {
 			/* save this brightness and dim the screen, fixes #328564 */
 			gpm_brightness_level_save (manager->priv->brightness,
@@ -991,6 +1019,9 @@ idle_changed_cb (GpmIdle    *idle,
 	case GPM_IDLE_MODE_SYSTEM:
 		gpm_debug ("Idle state changed: SYSTEM");
 
+		if (! gpm_manager_is_policy_timout_valid (manager, "timeout action")) {
+			return;
+		}
 		/* can only be hibernate or suspend */
 		gpm_manager_set_reason (manager, "the system state is idle");
 		manager_policy_do (manager, GPM_PREF_SLEEP_TYPE);
@@ -1024,9 +1055,9 @@ dpms_mode_changed_cb (GpmDpms    *dpms,
 
 	gpm_debug ("emitting dpms-mode-changed : %s", gpm_dpms_mode_to_string (mode));
 	g_signal_emit (manager,
-		       signals [DPMS_MODE_CHANGED],
-		       0,
-		       gpm_dpms_mode_to_string (mode));
+			signals [DPMS_MODE_CHANGED],
+			0,
+			gpm_dpms_mode_to_string (mode));
 }
 
 static void
@@ -1048,6 +1079,9 @@ static void
 power_button_pressed (GpmManager   *manager,
 		      gboolean	    state)
 {
+	if (! gpm_manager_is_policy_timout_valid (manager, "power button press")) {
+		return;
+	}
 	gpm_debug ("power button pressed");
 	gpm_manager_set_reason (manager, "the power button has been pressed");
 	manager_policy_do (manager, GPM_PREF_BUTTON_POWER);
@@ -1057,6 +1091,9 @@ static void
 suspend_button_pressed (GpmManager   *manager,
 			gboolean      state)
 {
+	if (! gpm_manager_is_policy_timout_valid (manager, "suspend button press")) {
+		return;
+	}
 	gpm_debug ("suspend button pressed");
 	gpm_manager_set_reason (manager, "the suspend button has been pressed");
 	manager_policy_do (manager, GPM_PREF_BUTTON_SUSPEND);
@@ -1066,6 +1103,9 @@ static void
 hibernate_button_pressed (GpmManager   *manager,
 			gboolean      state)
 {
+	if (! gpm_manager_is_policy_timout_valid (manager, "hibernate button press")) {
+		return;
+	}
 	gpm_debug ("hibernate button pressed");
 	gpm_manager_set_reason (manager, "the hibernate button has been pressed");
 	manager_policy_do (manager, GPM_PREF_BUTTON_HIBERNATE);
@@ -1077,20 +1117,28 @@ lid_button_pressed (GpmManager	 *manager,
 {
 	gboolean  on_ac;
 
+	if (manager->priv->lid_is_closed == state) {
+		gpm_debug ("duplicate lid change event");
+		return;
+	}
+
 	gpm_power_get_on_ac (manager->priv->power, &on_ac, NULL);
 
-	gpm_debug ("lid button changed: %d", state);
+	if (state) {
+		gpm_debug ("lid button CLOSED");
+	} else {
+		gpm_debug ("lid button OPENED");
+	}
 
-	/* We keep track of the lid state so we can do the 
+	/* We keep track of the lid state so we can do the
 	   lid close on battery action if the ac_adapter is removed when the laptop
 	   is closed. Fixes #331655 */
 	manager->priv->lid_is_closed = state;
 
 	if (state) {
-		/* Disable the screensaver, as we don't want this starting
+		/* Disable the fancy screensaver, as we don't want this starting
 		   when the lid is shut */
-		gpm_screensaver_inhibit_activation ("gnome-power-manager has "
-						    "detected that the lid is closed");
+		gpm_screensaver_enable_throttle (TRUE);
 		if (on_ac) {
 			gpm_debug ("Performing AC policy");
 			gpm_manager_set_reason (manager, "the lid has been closed on ac power");
@@ -1101,8 +1149,8 @@ lid_button_pressed (GpmManager	 *manager,
 			manager_policy_do (manager, GPM_PREF_BATTERY_BUTTON_LID);
 		}
 	} else {
-		/* re-enable the screensaver */
-		gpm_screensaver_allow_activation ();
+		/* Allow the fancy screensaver */
+		gpm_screensaver_enable_throttle (FALSE);
 
 		/* we turn the lid dpms back on unconditionally */
 		gpm_manager_unblank_screen (manager, NULL);
@@ -1117,6 +1165,9 @@ power_button_pressed_cb (GpmPower   *power,
 {
 	gpm_debug ("Button press event type=%s state=%d", type, state);
 
+	/* simulate user input */
+	gpm_screensaver_poke ();
+
 	if (strcmp (type, GPM_BUTTON_POWER) == 0) {
 		power_button_pressed (manager, state);
 
@@ -1155,6 +1206,9 @@ power_on_ac_changed_cb (GpmPower   *power,
 {
 	gpm_debug ("Setting on-ac: %d", on_ac);
 
+	/* simulate user input, to fix #333525 */
+	gpm_screensaver_poke ();
+
 	/* If we are on AC power we should show warnings again */
 	if (on_ac) {
 		gpm_debug ("Resetting last_primary_warning to NONE");
@@ -1171,7 +1225,7 @@ power_on_ac_changed_cb (GpmPower   *power,
 
 	/* We do the lid close on battery action if the ac_adapter is removed
 	   when the laptop is closed and on battery. Fixes #331655 */
-	if (! on_ac && manager->priv->lid_is_closed) {
+	if ( (! on_ac) && manager->priv->lid_is_closed) {
 		gpm_debug ("Doing battery policy when lid closed and power removed");
 		gpm_manager_set_reason (manager, "the lid has been closed, and the ac adapter removed");
 		manager_policy_do (manager, GPM_PREF_BATTERY_BUTTON_LID);
@@ -1198,7 +1252,13 @@ gpm_manager_get_warning_type (GpmPowerBatteryStatus *battery_status,
 			type = GPM_WARNING_LOW;
 		}
 	} else {
-		if (battery_status->percentage_charge <= BATTERY_ACTION_PERCENTAGE) {
+		if (battery_status->percentage_charge <= 0) {
+			type = GPM_WARNING_NONE;
+			gpm_warning ("Your hardware is reporting a percentage "
+				     "charge of %i, which is impossible. "
+				     "WARNING_ACTION will *not* be reported.",
+				     battery_status->percentage_charge);
+		} else if (battery_status->percentage_charge <= BATTERY_ACTION_PERCENTAGE) {
 			type = GPM_WARNING_ACTION;
 		} else if (battery_status->percentage_charge <= BATTERY_CRITICAL_PERCENTAGE) {
 			type = GPM_WARNING_CRITICAL;
@@ -1255,13 +1315,13 @@ manager_critical_action_do (GpmManager *manager)
 /* performs critical action is required, and displays notifications */
 static void
 battery_status_changed_primary (GpmManager	      *manager,
-			        GpmPowerBatteryKind    battery_kind,
+				 GpmPowerBatteryKind    battery_kind,
 				GpmPowerBatteryStatus *battery_status)
 {
 	GpmWarning  warning_type;
 	gboolean    show_notify;
-	char       *message = NULL;
-	char       *remaining = NULL;
+	char	*message = NULL;
+	char	*remaining = NULL;
 	const char *title = NULL;
 	gboolean    on_ac;
 
@@ -1269,7 +1329,7 @@ battery_status_changed_primary (GpmManager	      *manager,
 
 	/* If we are charging we should show warnings again as soon as we discharge again */
 	if (battery_status->is_charging) {
-		gpm_debug ("Resetting last_primary_warning to NONE");
+		gpm_debug ("Resetting warning to NONE as charging");
 		manager->priv->last_primary_warning = GPM_WARNING_NONE;
 	}
 
@@ -1277,10 +1337,10 @@ battery_status_changed_primary (GpmManager	      *manager,
 	 * 99 to 100 some laptops report this as charging, some as not-charging.
 	 * This is probably a race. This method should work for both cases. */
 	if (manager->priv->last_primary_percentage_change == 99 &&
-	    battery_status->percentage_charge == 100) {
+	    battery_status->percentage_charge == 100 &&
+	    ! manager->priv->done_notify_fully_charged) {
 		show_notify = gconf_client_get_bool (manager->priv->gconf_client,
 						     GPM_PREF_NOTIFY_BATTCHARGED, NULL);
-
 		if (show_notify) {
 			gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
 					      GPM_NOTIFY_TIMEOUT_SHORT,
@@ -1288,12 +1348,16 @@ battery_status_changed_primary (GpmManager	      *manager,
 					      NULL,
 					      _("Your battery is now fully charged"));
 		}
+		manager->priv->done_notify_fully_charged = TRUE;
+	}
+	/* We only re-enable the fully charged notification when the battery
+	   drops down to 95% as some batteries charge to 100% and then fluctuate
+	   from ~98% to 100%. See #338281 for details */
+	if (manager->priv->last_primary_percentage_change < 95) {
+		manager->priv->done_notify_fully_charged = FALSE;
 	}
 
-	if (battery_status->percentage_charge != manager->priv->last_primary_percentage_change) {
-		/* should we fire an event or something? */
-		manager->priv->last_primary_percentage_change = battery_status->percentage_charge;
-	}
+	manager->priv->last_primary_percentage_change = battery_status->percentage_charge;
 
 	if (! battery_status->is_discharging) {
 		gpm_debug ("%s is not discharging", battery_kind_to_string (battery_kind));
@@ -1317,6 +1381,10 @@ battery_status_changed_primary (GpmManager	      *manager,
 		const char *warning = NULL;
 		const char *action;
 
+		if (! gpm_manager_is_policy_timout_valid (manager, "critical action")) {
+			return;
+		}
+
 		/* we have to do different warnings depending on the policy */
 		action = gconf_client_get_string (manager->priv->gconf_client,
 						  GPM_PREF_BATTERY_CRITICAL, NULL);
@@ -1365,7 +1433,7 @@ battery_status_changed_primary (GpmManager	      *manager,
 							     GPM_PREF_NOTIFY_ACADAPTER, NULL);
 			if (show_notify) {
 				message = g_strdup_printf (_("The AC Power has been unplugged. "
-						             "The system is now using battery power."));
+							      "The system is now using battery power."));
 				timeout = GPM_NOTIFY_TIMEOUT_SHORT;
 			}
 		} else {
@@ -1392,7 +1460,7 @@ battery_status_changed_primary (GpmManager	      *manager,
 
 /* displays notifications */
 static void
-battery_status_changed_ups (GpmManager	          *manager,
+battery_status_changed_ups (GpmManager		   *manager,
 			    GpmPowerBatteryKind	   battery_kind,
 			    GpmPowerBatteryStatus *battery_status)
 {
@@ -1522,9 +1590,9 @@ battery_status_changed_misc (GpmManager	    	   *manager,
 }
 
 static void
-power_battery_status_changed_cb (GpmPower	       *power,
+power_battery_status_changed_cb (GpmPower		*power,
 				 GpmPowerBatteryKind	battery_kind,
-				 GpmManager	       *manager)
+				 GpmManager		*manager)
 {
 	GpmPowerBatteryStatus battery_status;
 
@@ -1685,6 +1753,11 @@ callback_gconf_key_changed (GConfClient *client,
 	} else if (strcmp (entry->key, GPM_PREF_CAN_HIBERNATE) == 0) {
 		gpm_manager_can_hibernate (manager, &enabled, NULL);
 		gpm_tray_icon_enable_hibernate (GPM_TRAY_ICON (manager->priv->tray_icon), enabled);
+
+	} else if (strcmp (entry->key, GPM_PREF_POLICY_TIMEOUT) == 0) {
+		manager->priv->suppress_policy_timeout =
+			gconf_client_get_int (manager->priv->gconf_client,
+				  	      GPM_PREF_POLICY_TIMEOUT, NULL);
 	}
 }
 
@@ -1693,6 +1766,9 @@ static void
 gpm_manager_tray_icon_hibernate (GpmManager   *manager,
 				 GpmTrayIcon  *tray)
 {
+	if (! gpm_manager_is_policy_timout_valid (manager, "hibernate signal")) {
+		return;
+	}
 	gpm_debug ("Received hibernate signal from tray icon");
 	gpm_manager_set_reason (manager, "user clicked hibernate from tray menu");
 	gpm_manager_hibernate (manager, NULL);
@@ -1702,6 +1778,9 @@ static void
 gpm_manager_tray_icon_suspend (GpmManager   *manager,
 			       GpmTrayIcon  *tray)
 {
+	if (! gpm_manager_is_policy_timout_valid (manager, "suspend signal")) {
+		return;
+	}
 	gpm_debug ("Received supend signal from tray icon");
 	gpm_manager_set_reason (manager, "user clicked suspend from tray menu");
 	gpm_manager_suspend (manager, NULL);
@@ -1718,6 +1797,14 @@ hal_battery_removed_cb (GpmHalMonitor *monitor,
 	tray_icon_update (manager);
 }
 
+static void
+tray_icon_destroyed (GtkObject *object, gpointer user_data)
+{
+	GpmManager *manager = user_data;
+
+	manager->priv->tray_icon = gpm_tray_icon_new ();
+}
+
 static void
 gpm_manager_init (GpmManager *manager)
 {
@@ -1774,6 +1861,8 @@ gpm_manager_init (GpmManager *manager)
 
 	gpm_debug ("creating new tray icon");
 	manager->priv->tray_icon = gpm_tray_icon_new ();
+	g_signal_connect (G_OBJECT (manager->priv->tray_icon), "destroy",
+			  G_CALLBACK (tray_icon_destroyed), manager);
 
 #if ACTIONS_MENU_ENABLED
 	gpm_manager_can_suspend (manager, &enabled, NULL);
@@ -1808,6 +1897,7 @@ gpm_manager_init (GpmManager *manager)
 	manager->priv->last_pda_warning = GPM_WARNING_NONE;
 	manager->priv->last_primary_warning = GPM_WARNING_NONE;
 	manager->priv->last_primary_percentage_change = 0;
+	manager->priv->done_notify_fully_charged = FALSE;
 
 	/* We don't want to be notified on coldplug if we are on battery power
 	   this should fix #332322 */
@@ -1822,12 +1912,21 @@ gpm_manager_init (GpmManager *manager)
 	   time_remaining is therefore inaccurate or just plain wrong. */
 	use_time = gconf_client_get_bool (manager->priv->gconf_client,
 					  GPM_PREF_USE_TIME_POLICY, NULL);
+	manager->priv->use_time_to_notify = use_time;
 	if (use_time) {
 		gpm_debug ("Using per-time notification policy");
 	} else {
 		gpm_debug ("Using percentage notification policy");
 	}
-	manager->priv->use_time_to_notify = use_time;
+
+	manager->priv->suppress_policy_timeout =
+		gconf_client_get_int (manager->priv->gconf_client,
+				      GPM_PREF_POLICY_TIMEOUT, NULL);
+	gpm_debug ("Using a supressed policy timeout of %i seconds",
+		   manager->priv->suppress_policy_timeout);
+
+	/* Pretend we just resumed when we start to let actions settle */
+	manager->priv->last_resume_event = time (NULL);
 }
 
 static void
@@ -1845,23 +1944,18 @@ gpm_manager_finalize (GObject *object)
 	if (manager->priv->gconf_client != NULL) {
 		g_object_unref (manager->priv->gconf_client);
 	}
-
 	if (manager->priv->dpms != NULL) {
 		g_object_unref (manager->priv->dpms);
 	}
-
 	if (manager->priv->idle != NULL) {
 		g_object_unref (manager->priv->idle);
 	}
-
 	if (manager->priv->power != NULL) {
 		g_object_unref (manager->priv->power);
 	}
-
 	if (manager->priv->brightness != NULL) {
 		g_object_unref (manager->priv->brightness);
 	}
-
 	if (manager->priv->tray_icon != NULL) {
 		g_object_unref (manager->priv->tray_icon);
 	}
