@@ -62,6 +62,13 @@ enum {
 	LAST_SIGNAL
 };
 
+typedef enum {
+	GPM_POWER_UNIT_MWH,
+	GPM_POWER_UNIT_CSR,
+	GPM_POWER_UNIT_PERCENT,
+	GPM_POWER_UNIT_UNKNOWN,
+} GpmPowerBatteryUnit;
+
 enum {
 	PROP_0,
 	PROP_ON_AC
@@ -83,7 +90,6 @@ typedef struct {
 	GpmPowerBatteryStatus	battery_status;
 	/* List of device udis */
 	GSList  *devices;
-
 } BatteryKindCacheEntry;
 
 typedef struct {
@@ -91,6 +97,12 @@ typedef struct {
 	int			charge_rate_previous;
 	GpmPowerBatteryKind	battery_kind;
 	GpmPowerBatteryStatus	battery_status;
+	char		       *product;
+	char		       *vendor;
+	char		       *technology;
+	GpmPowerBatteryUnit	unit;
+	char		       *serial;
+	char		       *model;
 } BatteryDeviceCacheEntry;
 
 /* Increasing this value will increase the damping effect
@@ -107,6 +119,7 @@ gpm_power_battery_status_set_defaults (GpmPowerBatteryStatus *status)
 	status->charge_rate = 0;
 	status->percentage_charge = 0;
 	status->remaining_time = 0;
+	status->capacity = 0;
 	status->is_rechargeable = FALSE;
 	status->is_present = FALSE;
 	status->is_charging = FALSE;
@@ -229,11 +242,40 @@ battery_device_cache_entry_update_all (BatteryDeviceCacheEntry *entry)
 			gpm_warning ("could not read your battery's remaining time");
 		}
 	}
+
+	/* calculate the batteries capacity if it is primary and present */
+	if (entry->battery_kind == GPM_POWER_BATTERY_KIND_PRIMARY && status->is_present) {
+		if (status->design_charge > 0 && status->last_full_charge > 0) {
+			if (status->design_charge != status->last_full_charge) {
+				float capacity;
+				capacity = status->design_charge / status->last_full_charge;
+				status->capacity = capacity * 100;
+			}
+		}
+	}
+
+	/* get other stuff we might need to know */
+	gpm_hal_device_get_string (udi, "info.product", &entry->product);
+	gpm_hal_device_get_string (udi, "info.vendor", &entry->vendor);
+	gpm_hal_device_get_string (udi, "battery.technology", &entry->technology);
+	gpm_hal_device_get_string (udi, "battery.serial", &entry->serial);
+	gpm_hal_device_get_string (udi, "battery.model", &entry->model);
+
+	if (entry->battery_kind == GPM_POWER_BATTERY_KIND_PRIMARY) {
+		/* true as not reporting, but charge_level */
+		entry->unit = GPM_POWER_UNIT_MWH;
+	} else if (entry->battery_kind == GPM_POWER_BATTERY_KIND_UPS) {
+		/* is this always correct? */
+		entry->unit = GPM_POWER_UNIT_PERCENT;
+	} else if (entry->battery_kind == GPM_POWER_BATTERY_KIND_MOUSE ||
+		   entry->battery_kind == GPM_POWER_BATTERY_KIND_KEYBOARD) {
+		entry->unit = GPM_POWER_UNIT_CSR;
+	}
 }
 
 static void
 battery_device_cache_entry_update_key (BatteryDeviceCacheEntry *entry,
-				       const char              *key)
+				       const char	      *key)
 {
 	GpmPowerBatteryStatus *status = &entry->battery_status;
 	char *udi = entry->udi;
@@ -311,6 +353,14 @@ battery_device_cache_entry_new_from_udi (const char *udi)
 	entry = g_new0 (BatteryDeviceCacheEntry, 1);
 
 	entry->udi = g_strdup (udi);
+
+	entry->product = NULL;
+	entry->vendor = NULL;
+	entry->technology = NULL;
+	entry->serial = NULL;
+	entry->model = NULL;
+	entry->unit = GPM_POWER_UNIT_UNKNOWN;
+
 	battery_device_cache_entry_update_all (entry);
 
 	return entry;
@@ -374,12 +424,14 @@ battery_kind_cache_debug_print (BatteryKindCacheEntry *entry)
 		   status->is_charging, status->charge_rate);
 	gpm_debug ("discharge  %i\tremaining  %i", 
 		   status->is_discharging, status->remaining_time);
+	gpm_debug ("capacity   %i", 
+		   status->capacity);
 }
 
 static void
-debug_print_type_cache_iter (gpointer               key,
+debug_print_type_cache_iter (gpointer	       key,
 			     BatteryKindCacheEntry *entry,
-			     gpointer              *user_data)
+			     gpointer	      *user_data)
 {
 	battery_kind_cache_debug_print (entry);
 }
@@ -415,8 +467,8 @@ battery_device_cache_find (GpmPower   *power,
 }
 
 static BatteryKindCacheEntry *
-battery_kind_cache_find (GpmPower   *power,
-			 GpmPowerBatteryKind battery_kind)
+battery_kind_cache_find (GpmPower		*power,
+			 GpmPowerBatteryKind	 battery_kind)
 {
 	BatteryKindCacheEntry *entry;
 
@@ -429,8 +481,196 @@ battery_kind_cache_find (GpmPower   *power,
 	return entry;
 }
 
+/** returns the number of devices of a specific kind */
+gint
+gpm_power_get_num_devices_of_kind (GpmPower		*power,
+				   GpmPowerBatteryKind	 battery_kind)
+{
+	BatteryKindCacheEntry *entry;
+	if (power->priv->battery_kind_cache == NULL) {
+		return 0;
+	}
+	entry = g_hash_table_lookup (power->priv->battery_kind_cache, &battery_kind);
+	if (entry == NULL) {
+		return 0;
+	}
+	return (g_slist_length (entry->devices));
+}
+
+/** frees the custom array type */
+void
+gpm_power_free_description_array (GArray *array)
+{
+	int a;
+	GpmPowerDescriptionItem *di;
+	for (a=0; a<array->len; a++) {
+		di = &g_array_index (array, GpmPowerDescriptionItem, a);
+		g_free (di->title);
+		g_free (di->value);
+	}
+	g_array_free (array, TRUE);
+}
+
+static const char *
+get_power_unit_suffix (GpmPowerBatteryUnit unit)
+{
+	const char *suffix;
+	if (unit == GPM_POWER_UNIT_MWH) {
+		suffix = "mWh";
+	} else if (unit == GPM_POWER_UNIT_PERCENT) {
+		suffix = "%";
+	} else if (unit == GPM_POWER_UNIT_CSR) {
+		suffix = "/7";
+	} else {
+		suffix = "?";
+	}
+	return suffix;
+}
+
+/** returns in a custom array the device parameters */
+GArray *
+gpm_power_get_description_array (GpmPower		*power,
+				 GpmPowerBatteryKind	 battery_kind,
+				 gint			 device_num)
+{
+	const char	        *udi;
+	const char		*suffix;
+	BatteryDeviceCacheEntry *device;
+	GpmPowerBatteryStatus	*status;
+	GpmPowerDescriptionItem  di;
+
+	GArray *array = g_array_new (FALSE, FALSE, sizeof (GpmPowerDescriptionItem));
+	BatteryKindCacheEntry *entry;
+	if (power->priv->battery_kind_cache == NULL) {
+		return NULL;
+	}
+	entry = g_hash_table_lookup (power->priv->battery_kind_cache, &battery_kind);
+	if (entry == NULL) {
+		return NULL;
+	}
+	if (device_num > g_slist_length (entry->devices) - 1) {
+		return NULL;
+	}
+
+	/* get the udi of the battery we are interested in */
+	udi = (const char *) g_slist_nth_data (entry->devices, device_num);
+
+	/* find the udi in the device cache */
+	device = battery_device_cache_find (power, udi);
+	if (device == NULL) {
+		return NULL;
+	}
+	status = &device->battery_status;
+
+	if (status->is_present == FALSE) {
+		di.title = g_strdup (_("Status:"));
+		di.value = g_strdup (_("Missing"));
+		g_array_append_vals (array, &di, 1);
+		return array;
+	}
+	if (gpm_power_battery_is_charged (status)) {
+		di.title = g_strdup (_("Status:"));
+		di.value = g_strdup (_("Charged"));
+		g_array_append_vals (array, &di, 1);
+	} else if (status->is_charging) {
+		di.title = g_strdup (_("Status:"));
+		di.value = g_strdup (_("Charging"));
+		g_array_append_vals (array, &di, 1);
+	} else if (status->is_discharging) {
+		di.title = g_strdup (_("Status:"));
+		di.value = g_strdup (_("Discharging"));
+		g_array_append_vals (array, &di, 1);
+	}
+	if (device->product) {
+		di.title = g_strdup (_("Product:"));
+		di.value = g_strdup (device->product);
+		g_array_append_vals (array, &di, 1);
+	}
+	if (device->vendor) {
+		di.title = g_strdup (_("Vendor:"));
+		di.value = g_strdup (device->vendor);
+		g_array_append_vals (array, &di, 1);
+	}
+	if (device->technology) {
+		di.title = g_strdup (_("Technology:"));
+		const char *technology;
+		if (strcasecmp (device->technology, "li-ion") == 0 ||
+		    strcasecmp (device->technology, "lion") == 0) {
+			technology = _("Lithium ion");
+		} else if (strcasecmp (device->technology, "pbac") == 0) {
+			technology = _("Lead acid");
+		} else {
+			gpm_warning ("Battery type %s not translated, please report!",
+				     device->technology);
+			technology = device->technology;
+		}
+		di.value = g_strdup (technology);
+		g_array_append_vals (array, &di, 1);
+	}
+	if (device->serial) {
+		di.title = g_strdup (_("Serial number:"));
+		di.value = g_strdup (device->serial);
+		g_array_append_vals (array, &di, 1);
+	}
+	if (device->model) {
+		di.title = g_strdup (_("Model:"));
+		di.value = g_strdup (device->model);
+		g_array_append_vals (array, &di, 1);
+	}
+	if (status->remaining_time > 0) {
+		di.title = g_strdup (_("Remaining time:"));
+		di.value = gpm_get_timestring (status->remaining_time);
+		g_array_append_vals (array, &di, 1);
+	}
+	if (status->percentage_charge > 0) {
+		di.title = g_strdup (_("Percentage charge:"));
+		di.value = g_strdup_printf ("%i%%", status->percentage_charge);
+		g_array_append_vals (array, &di, 1);
+	}
+	if (status->capacity > 0) {
+		const char *condition;
+		di.title = g_strdup (_("Capacity:"));
+		if (status->capacity > 99) {
+			condition = _("Very good condition");
+		} else if (status->capacity > 90) {
+			condition = _("Good condition");
+		} else {
+			condition = _("Poor condition");
+		}
+		di.value = g_strdup_printf ("%i%% (%s)", status->capacity, condition);
+		g_array_append_vals (array, &di, 1);
+	}
+	if (device->unit != GPM_POWER_UNIT_PERCENT) {
+		/* no point displaying these if we are measuring in percent */
+		suffix = get_power_unit_suffix (device->unit);
+		if (status->current_charge > 0) {
+			di.title = g_strdup (_("Current charge:"));
+			di.value = g_strdup_printf ("%i%s", status->current_charge, suffix);
+			g_array_append_vals (array, &di, 1);
+		}
+		if (status->last_full_charge > 0 &&
+		    status->design_charge != status->last_full_charge) {
+			di.title = g_strdup (_("Last full charge:"));
+			di.value = g_strdup_printf ("%i%s", status->last_full_charge, suffix);
+			g_array_append_vals (array, &di, 1);
+		}
+		if (status->design_charge > 0) {
+			di.title = g_strdup (_("Design charge:"));
+			di.value = g_strdup_printf ("%i%s", status->design_charge, suffix);
+			g_array_append_vals (array, &di, 1);
+		}
+		if (status->charge_rate > 0) {
+			di.title = g_strdup (_("Charge rate:"));
+			di.value = g_strdup_printf ("%imWh", status->charge_rate);
+			g_array_append_vals (array, &di, 1);
+		}
+	}
+
+	return array;
+}
+
 static void
-battery_kind_cache_update (GpmPower              *power,
+battery_kind_cache_update (GpmPower	         *power,
 			   BatteryKindCacheEntry *entry)
 {
 	GSList *l;
@@ -443,9 +683,9 @@ battery_kind_cache_update (GpmPower              *power,
 
 	/* iterate thru all the devices to handle multiple batteries */
 	for (l = entry->devices; l; l = l->next) {
-		const char              *udi;
+		const char	      *udi;
 		BatteryDeviceCacheEntry *device;
-		GpmPowerBatteryStatus		*device_status;
+		GpmPowerBatteryStatus	*device_status;
 
 		udi = (const char *)l->data;
 
@@ -531,9 +771,9 @@ battery_kind_cache_update (GpmPower              *power,
 }
 
 static void
-battery_kind_update_cache_iter (const char            *key,
+battery_kind_update_cache_iter (const char	      *key,
 				BatteryKindCacheEntry *entry,
-				GpmPower              *power)
+				GpmPower	      *power)
 {
 	battery_kind_cache_update (power, entry);
 }
@@ -551,7 +791,7 @@ battery_kind_cache_update_all (GpmPower *power)
 }
 
 static void
-battery_device_cache_add_device (GpmPower                *power,
+battery_device_cache_add_device (GpmPower		 *power,
 				 BatteryDeviceCacheEntry *entry)
 {
 	g_hash_table_insert (power->priv->battery_device_cache,
@@ -560,16 +800,22 @@ battery_device_cache_add_device (GpmPower                *power,
 }
 
 static void
-battery_device_cache_remove_device (GpmPower                *power,
+battery_device_cache_remove_device (GpmPower		    *power,
 				    BatteryDeviceCacheEntry *entry)
 {
 	g_hash_table_remove (power->priv->battery_device_cache,
 			     entry->udi);
+	g_free (entry->udi);
+	g_free (entry->product);
+	g_free (entry->vendor);
+	g_free (entry->technology);
+	g_free (entry->serial);
+	g_free (entry->model);
 }
 
 static void
-battery_kind_cache_add_device (GpmPower                *power,
-			       BatteryDeviceCacheEntry *device_entry)
+battery_kind_cache_add_device (GpmPower			*power,
+			       BatteryDeviceCacheEntry	*device_entry)
 {
 	BatteryKindCacheEntry *type_entry;
 
@@ -593,7 +839,7 @@ battery_kind_cache_add_device (GpmPower                *power,
 }
 
 static void
-battery_kind_cache_remove_device (GpmPower                *power,
+battery_kind_cache_remove_device (GpmPower		  *power,
 				  BatteryDeviceCacheEntry *entry)
 {
 	BatteryKindCacheEntry *type_entry;
@@ -622,8 +868,8 @@ power_get_summary_for_battery_kind (GpmPower		*power,
 {
 	BatteryKindCacheEntry *entry;
 	GpmPowerBatteryStatus *status;
-	const char            *type_desc = NULL;
-	char                  *timestring;
+	const char	    *type_desc = NULL;
+	char		  *timestring;
 
 	entry = battery_kind_cache_find (power, battery_kind);
 
@@ -740,7 +986,7 @@ gpm_power_get_status_summary (GpmPower *power,
 /* returns if the device was found in the cache */
 gboolean
 gpm_power_get_battery_status (GpmPower			 *power,
-			      GpmPowerBatteryKind         battery_kind,
+			      GpmPowerBatteryKind	 battery_kind,
 			      GpmPowerBatteryStatus      *battery_status)
 {
 	BatteryKindCacheEntry *entry;
@@ -927,21 +1173,13 @@ add_battery (GpmPower   *power,
 	 * where capacity is the ratio of the last_full capacity with that of
 	 * the design capacity. (#326740)
 	 */
-	if (entry->battery_kind == GPM_POWER_BATTERY_KIND_PRIMARY && status->is_present) {
-	        if (status->design_charge > 0 && status->last_full_charge > 0) {
-	                float capacity;
-	                capacity = status->design_charge / status->last_full_charge;
-	                gpm_debug ("Design: %i, Last full: %i, Capacity: %f",
-	                	   status->design_charge,
-	                	   status->last_full_charge,
-	                	   capacity);
-	                if (capacity < 0.5f) {
-	                        gpm_warning ("Your battery has a very low capacity, "
-	                                     "meaning that it may be old or broken. "
-	                                     "Battery life will be sub-optimal, "
-	                                     "and the time remaining may be incorrect.");
-	                }
-	        }
+	if (entry->battery_kind == GPM_POWER_BATTERY_KIND_PRIMARY) {
+		if (status->capacity > 0 && status->capacity < 0.5f) {
+			gpm_warning ("Your battery has a very low capacity, "
+				     "meaning that it may be old or broken. "
+				     "Battery life will be sub-optimal, "
+				     "and the time remaining may be incorrect.");
+		}
 	}
 	return TRUE;
 }
@@ -1090,8 +1328,8 @@ gpm_hash_new_device_cache (GpmPower *power)
 	gpm_debug ("creating cache");
 	power->priv->battery_device_cache = g_hash_table_new_full (g_str_hash,
 							   	   g_str_equal,
-							           g_free,
-							           NULL);
+								   g_free,
+								   NULL);
 }
 
 static void
@@ -1107,16 +1345,28 @@ gpm_hash_free_device_cache (GpmPower *power)
 	power->priv->battery_device_cache = NULL;
 }
 
-static void
-hal_disconnected_cb (GpmHalMonitor *monitor,
-		     GpmPower      *power)
+void
+gpm_power_dbus_name_owner_changed (GpmPower	*power,
+				   const char	*name,
+				   const char	*prev,
+				   const char	*new)
 {
-	/* We have to clear the caches, else the devices think they are
-	   initialised, and we segfault in various places. */
-	gpm_hash_free_kind_cache (power);
-	gpm_hash_new_kind_cache (power);
-	gpm_hash_free_device_cache (power);
-	gpm_hash_new_device_cache (power);
+	if (strcmp (name, HAL_DBUS_SERVICE) == 0) {
+		if (strlen (prev) != 0 && strlen (new) == 0 ) {
+			hal_stop_monitor (power->priv->hal_monitor);
+			/* We have to clear the caches, else the devices think they are
+			   initialised, and we segfault in various places. */
+			gpm_hash_free_kind_cache (power);
+			gpm_hash_free_device_cache (power);
+		}
+		if (strlen (prev) == 0 && strlen (new) != 0 ) {
+			/* Re-create the caches */
+			gpm_hash_new_kind_cache (power);
+			gpm_hash_new_device_cache (power);
+			hal_start_monitor (power->priv->hal_monitor);
+			battery_kind_cache_update_all (power);
+		}
+	}
 }
 
 static void
@@ -1137,8 +1387,6 @@ gpm_power_init (GpmPower *power)
 			  G_CALLBACK (hal_battery_added_cb), power);
 	g_signal_connect (power->priv->hal_monitor, "battery-removed",
 			  G_CALLBACK (hal_battery_removed_cb), power);
-	g_signal_connect (power->priv->hal_monitor, "hal-disconnected",
-			  G_CALLBACK (hal_disconnected_cb), power);
 
 	power->priv->battery_kind_cache = NULL;
 	power->priv->battery_device_cache = NULL;
